/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deployer.impl;

import java.io.File;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;

import org.mockito.Mock;
import org.ow2.util.archive.api.IFileArchive;
import org.ow2.util.archive.impl.FileArchiveImpl;
import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.IDeployerManager;
import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlanFragment;
import org.ow2.util.plan.deploy.deployable.api.DeploymentPlanDeployable;
import org.ow2.util.plan.deploy.deployable.impl.DeploymentPlanDeployableImpl;
import org.ow2.util.plan.deployer.impl.condition.ScriptConditionEvaluator;
import org.ow2.util.plan.deployer.impl.condition.script.RhinoScriptEngineFactory;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactoryManager;
import org.ow2.util.plan.reader.plan.PlanReaderImpl;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyZeroInteractions;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * A {@code DeploymentPlanDeployerTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class DeploymentPlanDeployerTestCase {

    private DeploymentPlanDeployer deployer;

    @Mock
    private IResourceFetcherFactoryManager resourceFetcherFactoryManager;
    @Mock
    private IResourceFetcher resourceFetcher;
    @Mock
    private IDeployerManager deployerManager;

    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);
        deployer = new DeploymentPlanDeployer();
        deployer.setPlanReader(new PlanReaderImpl());
        RhinoScriptEngineFactory rhino = new RhinoScriptEngineFactory();
        rhino.setScriptEngineFactory(getPlatformJavaScriptEngineFactory());
        ScriptConditionEvaluator evaluator = new ScriptConditionEvaluator();
        evaluator.setScriptEngineFactory(rhino);
        deployer.setConditionEvaluator(evaluator);
        deployer.setFetcherFactoryManager(resourceFetcherFactoryManager);
        deployer.setDeployerManager(deployerManager);

        when(resourceFetcherFactoryManager.getResourceFetcher(any(DeploymentPlanFragment.class))).thenReturn(resourceFetcher);
        when(resourceFetcher.getResource()).thenReturn(getFileResource("repositories.xml"));
    }

    private ScriptEngineFactory getPlatformJavaScriptEngineFactory() {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("JavaScript");
        return engine.getFactory();
    }


    @Test
    public void testDeployConditionalPlanOK() throws Exception {
        DeploymentPlanDeployable dpd = createDeploymentPlanDeployable("conditional-plan-ok.xml");
        deployer.deploy(dpd);

        assertTrue(deployer.isDeployed(dpd));

    }

    @Test
    public void testDeployConditionalPlanKO() throws Exception {
        DeploymentPlanDeployable dpd = createDeploymentPlanDeployable("conditional-plan-ko.xml");
        deployer.deploy(dpd);

        assertFalse(deployer.isDeployed(dpd));
        verifyZeroInteractions(resourceFetcherFactoryManager);
        verifyZeroInteractions(resourceFetcher);
    }

    @Test
    public void testDeployConditionalFragmentOK() throws Exception {
        DeploymentPlanDeployable dpd = createDeploymentPlanDeployable("conditional-fragment-ok.xml");
        deployer.deploy(dpd);

        assertTrue(deployer.isDeployed(dpd));

    }

    @Test
    public void testDeployConditionalFragmentKO() throws Exception {
        DeploymentPlanDeployable dpd = createDeploymentPlanDeployable("conditional-fragment-ko.xml");
        deployer.deploy(dpd);

        assertTrue(deployer.isDeployed(dpd));
        verifyZeroInteractions(resourceFetcherFactoryManager);
        verifyZeroInteractions(resourceFetcher);

    }


    private DeploymentPlanDeployable createDeploymentPlanDeployable(String planName) {
        return new DeploymentPlanDeployableImpl(createFileArchive(planName));
    }

    private IFileArchive createFileArchive(String planName) {
        return new FileArchiveImpl(getFileResource(planName));
    }

    private File getFileResource(String resource) {
        File basedir = new File(System.getProperty("basedir"));
        return new File(new File(basedir, "src/test/resources"), resource);
    }

}
