/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deployer.script;

import java.util.Hashtable;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;

import org.mockito.Mock;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.Version;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

/**
 * A {@code OSGiAwareScriptEngineFactoryTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class OSGiAwareScriptEngineFactoryTestCase {

    @Mock
    private BundleContext bundleContext;

    @Mock
    private Bundle bundle0;
    @Mock
    private Bundle bundle1;
    @Mock
    private Bundle bundle2;
    @Mock
    private Bundle bundle3;
    @Mock
    private Bundle bundle4;
    @Mock
    private Bundle bundle5;

    @Mock
    private ServiceReference<?> reference0;

    private OSGiAwareScriptEngineFactory factory;

    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);

        factory = new OSGiAwareScriptEngineFactory(bundleContext);
        factory.setScriptEngineFactory(getPlatformJavaScriptEngineFactory());

    }

    private ScriptEngineFactory getPlatformJavaScriptEngineFactory() {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("JavaScript");
        return engine.getFactory();
    }

    @Test
    public void testScriptEnvironmentVariables() throws Exception {

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundleContext"), bundleContext);

    }

    @Test
    public void testBundleInfoDefinition() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundleContext.getBundles()).thenReturn(new Bundle[] {bundle2, bundle1, bundle0});

        when(bundle0.getSymbolicName()).thenReturn("system");
        when(bundle0.getVersion()).thenReturn(new Version("1.0.0"));
        when(bundle1.getSymbolicName()).thenReturn("org.ow2.util");
        when(bundle1.getVersion()).thenReturn(new Version("1.0.0"));
        when(bundle2.getSymbolicName()).thenReturn("org.ow2.util");
        when(bundle2.getVersion()).thenReturn(new Version("2.0.0"));

        ScriptEngine engine = factory.getScriptEngine();
        assertNotNull(engine.eval("new BundleInfo(0)"));
        assertEquals(engine.eval("new BundleInfo(0).bundle"), bundle0);

        // Unknown Bundle
        assertNull(engine.eval("new BundleInfo(1).bundle"));

        // Bundles found using symbolic name
        assertEquals(engine.eval("new BundleInfo('system').bundle"), bundle0);
        assertEquals(engine.eval("new BundleInfo('org.ow2.util').bundle"), bundle2);
        assertEquals(engine.eval("new BundleInfo('org.ow2.util:1.0.0').bundle"), bundle1);
        assertEquals(engine.eval("new BundleInfo('org.ow2.util:2.0.0').bundle"), bundle2);

        assertNull(engine.eval("new BundleInfo('unavailable-name').bundle"));
        assertNull(engine.eval("new BundleInfo('unavailable-name:1.0.0').bundle"));
    }

    @Test
    public void testBundleFunctionDefinition() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).bundle"), bundle0);
    }

    @Test
    public void testBundleIsUninstalled() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertTrue((Boolean) engine.eval("bundle(0).uninstalled"));
        assertFalse((Boolean) engine.eval("bundle(1).uninstalled"));
        assertFalse((Boolean) engine.eval("bundle(2).uninstalled"));
        assertFalse((Boolean) engine.eval("bundle(3).uninstalled"));
        assertFalse((Boolean) engine.eval("bundle(4).uninstalled"));
        assertFalse((Boolean) engine.eval("bundle(5).uninstalled"));

    }

    private void setMockBundleStates() {
        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundleContext.getBundle(1)).thenReturn(bundle1);
        when(bundleContext.getBundle(2)).thenReturn(bundle2);
        when(bundleContext.getBundle(3)).thenReturn(bundle3);
        when(bundleContext.getBundle(4)).thenReturn(bundle4);
        when(bundleContext.getBundle(5)).thenReturn(bundle5);
        when(bundle0.getState()).thenReturn(Bundle.UNINSTALLED);
        when(bundle1.getState()).thenReturn(Bundle.INSTALLED);
        when(bundle2.getState()).thenReturn(Bundle.RESOLVED);
        when(bundle3.getState()).thenReturn(Bundle.STARTING);
        when(bundle4.getState()).thenReturn(Bundle.STOPPING);
        when(bundle5.getState()).thenReturn(Bundle.ACTIVE);
    }

    @Test
    public void testBundleIsInstalled() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertFalse((Boolean) engine.eval("bundle(0).installed"));
        assertTrue((Boolean) engine.eval("bundle(1).installed"));
        assertTrue((Boolean) engine.eval("bundle(2).installed"));
        assertTrue((Boolean) engine.eval("bundle(3).installed"));
        assertTrue((Boolean) engine.eval("bundle(4).installed"));
        assertTrue((Boolean) engine.eval("bundle(5).installed"));

    }

    @Test
    public void testBundleIsResolved() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertFalse((Boolean) engine.eval("bundle(0).resolved"));
        assertFalse((Boolean) engine.eval("bundle(1).resolved"));
        assertTrue((Boolean) engine.eval("bundle(2).resolved"));
        assertTrue((Boolean) engine.eval("bundle(3).resolved"));
        assertTrue((Boolean) engine.eval("bundle(4).resolved"));
        assertTrue((Boolean) engine.eval("bundle(5).resolved"));

    }

    @Test
    public void testBundleIsStarting() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertFalse((Boolean) engine.eval("bundle(0).starting"));
        assertFalse((Boolean) engine.eval("bundle(1).starting"));
        assertFalse((Boolean) engine.eval("bundle(2).starting"));
        assertTrue((Boolean) engine.eval("bundle(3).starting"));
        assertFalse((Boolean) engine.eval("bundle(4).starting"));
        assertFalse((Boolean) engine.eval("bundle(5).starting"));

    }

    @Test
    public void testBundleIsStopping() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertFalse((Boolean) engine.eval("bundle(0).stopping"));
        assertFalse((Boolean) engine.eval("bundle(1).stopping"));
        assertFalse((Boolean) engine.eval("bundle(2).stopping"));
        assertFalse((Boolean) engine.eval("bundle(3).stopping"));
        assertTrue((Boolean) engine.eval("bundle(4).stopping"));
        assertFalse((Boolean) engine.eval("bundle(5).stopping"));

    }

    @Test
    public void testBundleIsActive() throws Exception {

        setMockBundleStates();

        ScriptEngine engine = factory.getScriptEngine();
        assertFalse((Boolean) engine.eval("bundle(0).active"));
        assertFalse((Boolean) engine.eval("bundle(1).active"));
        assertFalse((Boolean) engine.eval("bundle(2).active"));
        assertFalse((Boolean) engine.eval("bundle(3).active"));
        assertFalse((Boolean) engine.eval("bundle(4).active"));
        assertTrue((Boolean) engine.eval("bundle(5).active"));

    }


    @Test
    public void testBundleExists() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);

        ScriptEngine engine = factory.getScriptEngine();
        assertTrue((Boolean) engine.eval("bundle(0).exists"));
        assertFalse((Boolean) engine.eval("bundle(1).exists"));

    }

    @Test
    public void testBundleHeaders() throws Exception {

        Hashtable<String, String> p = new Hashtable<String, String>();
        p.put("titi", "toto");
        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getHeaders()).thenReturn(p);

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).headers['titi']"), "toto");
        assertNull(engine.eval("bundle(0).headers['not-defined']"));

    }

    @Test
    public void testBundleSymbolicName() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getSymbolicName()).thenReturn("org.ow2.util");

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).symbolicName"), "org.ow2.util");

    }

    @Test
    public void testBundleId() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getBundleId()).thenReturn(0l);

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).bundleId"), 0l);

    }

    @Test
    public void testBundleState() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getState()).thenReturn(Bundle.ACTIVE);

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).state"), Bundle.ACTIVE);

    }

    @Test
    public void testBundleLocation() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getLocation()).thenReturn("here");

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).location"), "here");

    }

    @Test
    public void testBundleLastModified() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getLastModified()).thenReturn(1024l);

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).lastModified"), 1024l);

    }

    @Test
    public void testBundleVersion() throws Exception {

        when(bundleContext.getBundle(0)).thenReturn(bundle0);
        when(bundle0.getVersion()).thenReturn(new Version("1.0.0"));

        ScriptEngine engine = factory.getScriptEngine();
        assertEquals(engine.eval("bundle(0).version"), "1.0.0");

    }

    @Test
    public void testServiceReference() throws Exception {

        when(bundleContext.getAllServiceReferences("java.lang.Runnable", null))
                .thenReturn(new ServiceReference<?>[]{reference0});

        ScriptEngine engine = factory.getScriptEngine();
        assertTrue((Boolean) engine.eval("service('java.lang.Runnable').available"));
        assertFalse((Boolean) engine.eval("service('java.util.Iterable').available"));

    }

    @Test
    public void testServiceReferenceWithFiltering() throws Exception {

        when(bundleContext.getAllServiceReferences("java.lang.Runnable", "(a=b)"))
                .thenReturn(new ServiceReference<?>[]{reference0});

        ScriptEngine engine = factory.getScriptEngine();

        assertTrue((Boolean) engine.eval("service('java.lang.Runnable').filter('(a=b)').available"));
        assertFalse((Boolean) engine.eval("service('java.lang.Runnable').filter('(b=c)').available"));

    }

    @Test
    public void testServiceReferenceWithOnlyFiltering() throws Exception {

        when(bundleContext.getAllServiceReferences(null, "(a=b)"))
                .thenReturn(new ServiceReference<?>[]{reference0});

        ScriptEngine engine = factory.getScriptEngine();

        assertTrue((Boolean) engine.eval("service(null).filter('(a=b)').available"));
        assertTrue((Boolean) engine.eval("service().filter('(a=b)').available"));

    }
}
