/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deployer.test;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

import java.io.File;

import org.ow2.util.archive.api.IFileArchive;
import org.ow2.util.archive.impl.ArchiveManager;
import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.DeployerException;
import org.ow2.util.ee.deploy.impl.deployer.DeployerManager;
import org.ow2.util.ee.deploy.impl.helper.DeployableHelper;
import org.ow2.util.plan.bindings.repository.Repository;
import org.ow2.util.ee.deploy.api.deployable.factory.IFileDeployableFactoryManager;
import org.ow2.util.ee.deploy.api.deployable.factory.XmlFileDeployableFactory;
import org.ow2.util.plan.deploy.deployable.impl.RepositoryDeployableImpl;
import org.ow2.util.ee.deploy.impl.deployable.factory.DefaultFileDeployableFactoryManager;
import org.ow2.util.ee.deploy.impl.deployable.factory.XmlFileDeployableFactoryImpl;
import org.ow2.util.plan.deployer.api.IRepositoryDeployer;
import org.ow2.util.plan.deployer.impl.DefaultRepositoryDeployerImpl;
import org.ow2.util.plan.reader.repository.RepositoryDataReaderImpl;
import org.ow2.util.plan.repository.impl.DefaultRepositoryManager;
import org.testng.annotations.Test;
import org.testng.annotations.BeforeTest;

/**
 * Test class for the repository deployer.
 * @author mleduque
 */
public class RepositoryDeployTest {

    private static final String REPOSITORIES_XML_PATH = "/repositories.xml";
    private static final String REPOSITORIES2_XML_PATH = "/repositories2.xml";
    private static final String REPOSITORIES3_XML_PATH = "/repositories3.xml";

    private static final String REPOSITORY_NS =
    "http://jonas.ow2.org/ns/deployment-plan/repositories/1.0";

    private XmlFileDeployableFactory factory = null;
    private IFileDeployableFactoryManager manager = null;

    private DeployerManager deployerManager = null;
    private IRepositoryDeployer repositoryDeployer = null;

    private IDeployable[] deployable = new IDeployable[3];
    private IFileArchive[] repositoryArchives = new IFileArchive[3];

    @BeforeTest(alwaysRun=true)
    public void setup() throws Exception {
        // ClassCastException
        factory = new XmlFileDeployableFactoryImpl();
        manager = new DefaultFileDeployableFactoryManager();
        manager.addFileDeployableFactory(factory);
        DeployableHelper.setFileDeployableFactory(manager);

        factory.registerFileDeployable(
                RepositoryDeployableImpl.class,
                REPOSITORY_NS);

        repositoryDeployer = new DefaultRepositoryDeployerImpl();
        repositoryDeployer.setReader(new RepositoryDataReaderImpl());

        deployerManager = new DeployerManager();
        deployerManager.register(repositoryDeployer);


        }

    @Test
    public void prepareRepository1() throws Exception {
        File f = new File(RepositoryDeployTest.class.getResource(REPOSITORIES_XML_PATH).getFile());
        repositoryArchives[0] = (IFileArchive)ArchiveManager.getInstance().getArchive(f);
        deployable[0] = DeployableHelper.getDeployable(repositoryArchives[0]);
        assertNotNull(deployable[0]);
        assertEquals(deployable[0].getClass(), RepositoryDeployableImpl.class);
    }

    @Test
    public void prepareRepository2() throws Exception {
        File f = new File(RepositoryDeployTest.class.getResource(REPOSITORIES2_XML_PATH).getFile());
        repositoryArchives[1] = (IFileArchive)ArchiveManager.getInstance().getArchive(f);
        deployable[1] = DeployableHelper.getDeployable(repositoryArchives[1]);
        assertNotNull(deployable[1]);
        assertEquals(deployable[1].getClass(), RepositoryDeployableImpl.class);
    }

    @Test
    public void prepareRepository3() throws Exception {
        File f = new File(RepositoryDeployTest.class.getResource(REPOSITORIES3_XML_PATH).getFile());
        repositoryArchives[2] = (IFileArchive)ArchiveManager.getInstance().getArchive(f);
        deployable[2] = DeployableHelper.getDeployable(repositoryArchives[2]);
        assertNotNull(deployable[2]);
        assertEquals(deployable[2].getClass(), RepositoryDeployableImpl.class);
    }

    @Test(dependsOnMethods = {"prepareRepository1", "prepareRepository2", "prepareRepository3"})
    public void deployRepository() throws Exception {
        repositoryDeployer.deploy(deployable[0]);
        assertTrue(repositoryDeployer.isDeployed(deployable[0]));
        repositoryDeployer.undeploy(deployable[0]);
        assertFalse(repositoryDeployer.isDeployed(deployable[0]));
    }

    @Test(dependsOnMethods = {"deployRepository"})
    public void deployWithManager() throws Exception {
        deployerManager.deploy(deployable[0]);
        //assertTrue(deployerManager.isDeployed(deployable));
        deployerManager.undeploy(deployable[0]);
        //assertFalse(deployerManager.isDeployed(deployable));
    }

    public int countRepositories() {
        int i = 0;
        for (Repository repository : DefaultRepositoryManager.getInstance()) {
            i++;
        }
        return i;
    }

    @Test(dependsOnMethods = {"deployWithManager"})
    public void checkRepositoriesCount() throws Exception {
        deployerManager.deploy(deployable[0]);
        deployerManager.deploy(deployable[1]);
        assertEquals(countRepositories(), 5);
    }

    @Test(dependsOnMethods = {"checkRepositoriesCount"},
          expectedExceptions = {DeployerException.class})
    public void checkFailureOnIdCollision() throws Exception {
        deployerManager.deploy(deployable[2]);
    }

    @Test(dependsOnMethods = {"checkFailureOnIdCollision"})
    public void checkCountAfterFailure() throws Exception {
        assertEquals(countRepositories(), 5);
    }

    @Test(dependsOnMethods = {"checkCountAfterFailure"})
    public void checkUndeploy() throws Exception {
        deployerManager.undeploy(deployable[1]);
        assertEquals(countRepositories(), 3);
        deployerManager.undeploy(deployable[0]);
        assertEquals(countRepositories(), 0);
    }
}
