/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.fetcher.impl;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlanFragment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.bindings.repository.Repository;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactory;
import org.ow2.util.plan.fetcher.api.exceptions.FetcherFactoryException;
import org.ow2.util.plan.repository.api.IRepositoryManager;
import org.ow2.util.plan.repository.impl.CustomRepositoryManager;

/**
 *
 * @author Mickaël LEDUQUE
 */
public abstract class AbsResourceFetcherFactoryImpl implements IResourceFetcherFactory {

    /**
     * The class used for fetcher instantiation.
     */
    private Class<? extends IResourceFetcher> fetcherClass = null;

    /**
     * The repository manager that will be used.
     */
    private IRepositoryManager repositoryManager = null;

    /**
     * The local directory used to store the local copy of the resource.
     */
    protected File localRepositoriesBaseDir = null;

    /**
     * List of registered resource fetchers.
     */
    protected List<IResourceFetcher> resourceFetchers = null;

    /**
     * Default constructor.
     */
    public AbsResourceFetcherFactoryImpl() {
        this.resourceFetchers = new ArrayList<IResourceFetcher>();
    }

    /**
     * {@inheritDoc}
     */
    public IResourceFetcher getResourceFetcher(final DeploymentPlanFragment fragment) throws FetcherFactoryException {
        Class<? extends DeploymentPlanFragment> fragmentClass = fragment.getClass();
        if (fragmentClass != this.getSupportedClass()) {
            throw new FetcherFactoryException(
                    "This Repository Fetcher Factory can't create fetchers for repository type "
                    + fragmentClass);
        }
        String repository = fragment.getRepositoryRef();

        IResourceFetcher fetcher = null;
        try {
            Constructor<? extends IResourceFetcher> constructor = fetcherClass.getConstructor();
            fetcher = constructor.newInstance();
        } catch (Exception e) {
            throw new FetcherFactoryException(e);
        }

        if (repository == null) {
            /* use the whole repository manager */
            fetcher.setRepositoryManager(this.repositoryManager);
        } else {
            Repository rep = this.repositoryManager.getRepositoryById(repository);
            if (rep == null) {
                throw new FetcherFactoryException("There is no repository with id {0}"
                        + repository);
            }
            IRepositoryManager newManager = new CustomRepositoryManager(rep);
            fetcher.setRepositoryManager(newManager);
        }

        fetcher.setLocalRepositoriesBaseDir(this.localRepositoriesBaseDir);
        try {
            fetcher.setDeployment(fragment);
        } catch (InvalidDeploymentException e) {
            throw new FetcherFactoryException(e);
        }

        resourceFetchers.add(fetcher);

        return fetcher;
    }

    /**
     * {@inheritDoc}
     */
    public void setFetcherConcreteClass(final Class<? extends IResourceFetcher> clazz) {
        this.fetcherClass = clazz;
    }

    /**
     * {@inheritDoc}
     */
    public Class<? extends IResourceFetcher> getFetcherConcreteClass() {
        return this.fetcherClass;
    }

    /**
     * {@inheritDoc}
     */
    public void setRepositoryManager(final IRepositoryManager repositoryManager) {
        this.repositoryManager = repositoryManager;
    }

    /**
     * {@inheritDoc}
     */
    public IRepositoryManager getRepositoryManager() {
        return this.repositoryManager;
    }

    /**
     * {@inheritDoc}
     */
    public void setLocalRepositoriesBaseDir(final File localRepositoriesBaseDir) {
        for (IResourceFetcher resourceFetcher: resourceFetchers) {
            resourceFetcher.setLocalRepositoriesBaseDir(localRepositoriesBaseDir);
        }
        this.localRepositoriesBaseDir = localRepositoriesBaseDir;
    }
}
