/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.fetcher.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlanFragment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactory;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactoryManager;
import org.ow2.util.plan.fetcher.api.exceptions.FetcherFactoryException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceNotFoundException;

/**
 * @author Mickaël LEDUQUE
 */
public class ResourceFetcherFactoryManagerImpl implements IResourceFetcherFactoryManager {

    /**
     * The resource fetcher factories.
     */
    private Map<Class<? extends DeploymentPlanFragment>, IResourceFetcherFactory> factories = new HashMap<Class<? extends DeploymentPlanFragment>, IResourceFetcherFactory>();

    /**
     * Default constructor.
     */
    public ResourceFetcherFactoryManagerImpl() {
    }

    /**
     * {@inheritDoc}
     */
    public void addResourceFetcherFactory(final IResourceFetcherFactory factory) {
        factories.put(factory.getSupportedClass(), factory);
    }

    /**
     * {@inheritDoc}
     */
    public void removeResourceFetcherFactory(final IResourceFetcherFactory factory) {
        factories.remove(factory.getSupportedClass());
    }

    /**
     * {@inheritDoc}
     * @throws FetcherFactoryException
     */
    public IResourceFetcher getResourceFetcher(final DeploymentPlanFragment deployment) throws ResourceNotFoundException,
            InvalidDeploymentException, FetcherFactoryException {
        if (deployment == null) {
            throw new InvalidDeploymentException("null deployment");
        }
        IResourceFetcherFactory factory = this.findFetcherFactoryForDeployment(deployment);
        if (factory == null) {
            throw new FetcherFactoryException("No resource fetcher factory for deployment class " + deployment.getClass());
        }
        IResourceFetcher fetcher = factory.getResourceFetcher(deployment);
        return fetcher;
    }

    public IResourceFetcherFactory findFetcherFactoryForDeployment(final DeploymentPlanFragment fragment) {
        IResourceFetcherFactory factory = factories.get(fragment.getClass());
        if (factory == null) {
            for (IResourceFetcherFactory loopFactory : factories.values()) {
                if (loopFactory.getSupportedClass().isAssignableFrom(fragment.getClass())) {
                    factory = loopFactory;
                    break;
                }
            }
        }
        return factory;
    }

    /**
     * {@inheritDoc}
     */
    public Iterator<IResourceFetcherFactory> iterator() {
        return this.factories.values().iterator();
    }
}
