/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.fetcher.impl.maven2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ow2.util.base64.Base64;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.w3c.dom.Document;

/**
 * Maven2 Utilities.
 * @author Francois Fornaciari
 */
public final class Maven2Utils {

    /**
     * The logger.
     */
    private static final Log LOGGER = LogFactory.getLog(Maven2Utils.class);

    /**
     * Private empty constructor.
     */
    private Maven2Utils() {
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param maven2Version Maven2 version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource location
     */
    public static String getMaven2Location(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type) {

        // Creates a File from artifact info
        String appendedClassifier = "";
        if (classifier != null) {
            // Handle classifier if any
            appendedClassifier = appendedClassifier.concat("-").concat(classifier);
        }

        String fileExtension = null;
        if ("deployment-plan".equals(type)) {
            fileExtension = "xml";
        } else {
            fileExtension = type;
        }

        String path = constructDirectoryPath(groupId, artifactId, version);
        String location = repository.concat("/").concat(
                path.concat("/").concat(artifactId).concat("-").concat(maven2Version).concat(appendedClassifier).concat(".")
                        .concat(fileExtension));
        return location;
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param maven2Version Maven2 version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource file
     */
    public static File getMaven2File(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type) {
        return new File(getMaven2Location(repository, groupId, artifactId, version, maven2Version, classifier, type));
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param maven2Version Maven2 version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource URL
     * @throws MalformedURLException if URL is malformed
     */
    public static URL getMaven2URL(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type)
            throws MalformedURLException {
        return new URL(getMaven2Location(repository, groupId, artifactId, version, maven2Version, classifier, type));
    }

    /**
     * Read the checksum from a given input stream.
     * @param inputStream The stream to read
     * @return The checksum of a given input stream
     * @throws IOException If the reading of the checksum fails
     */
    public static String readChecksum(final InputStream inputStream) throws IOException {
        String checksum = "";

        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        try {
            inputStreamReader = new InputStreamReader(inputStream);
            bufferedReader = new BufferedReader(inputStreamReader);

            // Read the checksum
            checksum = bufferedReader.readLine().trim();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.reset();
                    // Close buffered reader
                    bufferedReader.close();
                } catch (IOException e) {
                    // Nothing to do
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.read();
                    // Close input stream reader
                    inputStreamReader.close();
                } catch (IOException e) {
                    // Nothing to do
                }
            }
        }

        // Return the first entry of the file
        return checksum.split(" ")[0];
    }

    /**
     * Returns the artifact version to be passed to the artifact URL builder May
     * be a timestamped version for SNAPSHOT artifacts.
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param timeout the tiemout to wait before canceling the request
     * @return The artifact version to be passed to the artifact URL builder
     */
    public static String getMaven2Version(final String repository, final String groupId, final String artifactId,
            final String version, final int timeout) {
        String maven2Version = version;

        if (version.endsWith("SNAPSHOT")) {
            String path = constructDirectoryPath(groupId, artifactId, version);
            InputStream inputStream = null;
            URL mavenMetadataURL = null;
            try {
                String repositoryVal = repository;
                if (!repository.endsWith("/")) {
                    repositoryVal = repository.concat("/");
                }
                mavenMetadataURL = new URL(repositoryVal.concat(path.concat("/")).concat("maven-metadata.xml"));
                URLConnection connection = mavenMetadataURL.openConnection();
                addAuthenticationOnURL(connection);
                connection.setReadTimeout(timeout);
                inputStream = connection.getInputStream();

                DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder parser = documentBuilderFactory.newDocumentBuilder();
                Document document = parser.parse(inputStream);

                // Read timestamped version
                String timestamp = document.getElementsByTagName("timestamp").item(0).getTextContent();
                String buildNumber = document.getElementsByTagName("buildNumber").item(0).getTextContent();

                maven2Version = version.replace("SNAPSHOT", timestamp.concat("-").concat(buildNumber));
            } catch (FileNotFoundException e) {
                // Do nothing
            } catch (Exception e) {
                LOGGER.debug("Unable to analyze metadata.xml file ''{0}''", mavenMetadataURL, e);
                // If the maven-metadata.xml file doesn't contain the timestamp
                // and the buildNumber elements
                // In both cases we have to return the SNAPSHOT version
            } finally {
                if (inputStream != null) {
                    try {
                        // Close input stream
                        inputStream.close();
                    } catch (IOException e) {
                        // Nothing to do
                    }
                }
            }
        }

        return maven2Version;
    }

    /**
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @return transform the artifact coordinates into a m2 repository path
     */
    private static String constructDirectoryPath(final String groupId, final String artifactId, final String version) {
        // Transform groupId '.' into '/'
        String transformed = groupId.replace('.', '/');
        return transformed.concat("/").concat(artifactId).concat("/").concat(version);
    }

    /**
     * Add the Basic Authorization if defined in the URL as user info.
     * @param urlConnection the URL connection to update
     */
    protected static void addAuthenticationOnURL(final URLConnection urlConnection) {

        // Get userinfo data
        String userInfo = urlConnection.getURL().getUserInfo();

        // No data, skip
        if (userInfo == null) {
            return;
        }

        // Get username and password
        String username = null;
        String password = null;

        int i = userInfo.indexOf(':');
        if (i >= 0) {
            username = userInfo.substring(0, i);
            password = userInfo.substring(i + 1);
        } else {
            // all the content is username
            username = userInfo;
        }

        // Add token which is username:password
        String token = username.concat(":").concat(password);

        // And then encode the token with base64
        String auth = "Basic ".concat(String.valueOf(Base64.encode(token.getBytes())));

        // Update URL connection
        urlConnection.setRequestProperty("Authorization", auth.toString());


    }
}
