/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.bindings.repository;

import javax.xml.bind.annotation.XmlTransient;

import org.ow2.util.substitution.ISubstitutionEngine;
import org.ow2.util.substitution.engine.DefaultSubstitutionEngine;
import org.ow2.util.substitution.resolver.ChainedResolver;
import org.ow2.util.substitution.resolver.PropertiesResolver;
import org.ow2.util.substitution.resolver.RecursiveResolver;

/**
 * Actual Repository class that will be used by the JAXB2 unmarshaller.
 * @author Mickael LEDUQUE
 */
public class ExtendedRepository extends Repository {

    /**
     * Property substitution engine.
     */
    @XmlTransient
    private ISubstitutionEngine substitutionEngine;

    /**
     * URL has been updated.
     */
    private boolean updatedURL = false;

    /**
     * Default constructor.
     */
    public ExtendedRepository() {
        super();
        this.substitutionEngine = createSubstitutionEngine();
    }

    /**
     * Build a new Substitution engine.
     * @return the substituion engine that will be used for variable value resolution.
     */
    protected ISubstitutionEngine createSubstitutionEngine() {

        // List of resolvers
        ChainedResolver resolver = new ChainedResolver();

        // Create Engine
        DefaultSubstitutionEngine engine = new DefaultSubstitutionEngine();
        engine.setMarkerChar('$');
        engine.setOpeningChar('{');
        engine.setEndingChar('}');
        engine.setResolver(new RecursiveResolver(engine, resolver));

        // Add the system properties as resolver
        resolver.getResolvers().add(new PropertiesResolver(System.getProperties()));

        return engine;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("id=");
        String id = this.getId();
        if (id != null) {
            sb.append(id);
        } else {
            sb.append("<not set>");
        }
        sb.append(":type=");
        sb.append(this.getType());
        sb.append(":url=");
        sb.append(this.getUrl());

        return sb.toString();
    }

    /**
     * If the value contains some properties, resolve them and update them.
     * @return URL with the updated value if any
     */
    @Override
    public String getUrl() {
        if (!updatedURL) {
            this.url = substitutionEngine.substitute(this.url);
            updatedURL = true;
        }
        return super.getUrl();
    }


}
